#include "worker.h"

WorkerThread::WorkerThread(QWidget *parent) : QThread(parent)
{
    m_fDes = -1;
    m_eof = 0;
    m_TTY = 0;
}

void WorkerThread::run()
{
    string	dataReadFromChild;
    ssize_t	readResult;
    char	buffer[BUFFER_SIZE + 1];
    int		status;
    int     data_cued = 0;

    int pollResult = 0;

    while (m_eof == 0) {

        errno = 0;
        pollResult = poll(&m_pollOutput, 1, POLL_TIMEOUT);
        if (pollResult < 0 && errno == POLLERR) {
            FAIL("The file handle from parent is broken.");
            return;
        }

        switch (readResult = read(m_fDes, buffer, BUFFER_SIZE)) {
        case 0: /* End-of-File. */

            m_eof = 1;
            emit child_sent(QString((char *) dataReadFromChild.c_str()));
            ASSERT_IS(m_TTY, waitpid(m_TTY, & status, 0));
            emit child_sent(QString("The shell has exited correctly."));
            return;

        case -1:
            if (errno == EINTR) {
                errno = 0;
                break;
            } else if (errno == EAGAIN || errno == EWOULDBLOCK) {
                errno = 0;

                if (data_cued == 0)
                    continue;

                emit child_sent(QString((char *) dataReadFromChild.c_str()));
                dataReadFromChild = "";
                data_cued = 0;
                break;
            } else {

                FAIL("read() failed in parent process.");
                return;
            }

        default:
            buffer[BUFFER_SIZE] = '\0';
            try {
                dataReadFromChild . append(buffer, readResult);
                data_cued += readResult;
            } catch (std::length_error) {
                dataReadFromChild = "";
                data_cued = 0;
            }

            break;
        }
    } /* while (true) */
}

void WorkerThread::setHandle(int fDes)
{
    m_fDes = fDes;

    fcntl( fDes, F_SETFL, fcntl(fDes, F_GETFL) | O_NONBLOCK );
}

void WorkerThread::setTTY(pid_t tty)
{
    m_TTY = tty;
}

